@@ -27,6 +27,14 @@ Content = Union[bytes, bytearray, str, Path, io.IOBase]
 Queryparam = Union[str, List[str]]
 
 
+def datetimestr(d: Optional[Union[str, datetime]]) -> Optional[str]:
+    if isinstance(d, datetime):
+        if not d.tzinfo:
+            d = d.astimezone()
+        return d.isoformat()
+    return d
+
+
 def dictstrip(d):
     """Given a dict, return the dict with keys mapping to falsey values removed."""
     return {k: v for k, v in d.items() if v}
@@ -118,6 +126,11 @@ def _(content, find_content_type=False):
     return b64encode(raw).decode(), content_type
 
 
+class EmptyRequestError(ValueError):
+    """An EmptyRequestError is raised if the request body is empty when expected not to be empty."""
+    pass
+
+
 class ClientException(Exception):
     """A ClientException is raised if the client refuses to
     send request due to incorrect usage or bad request data."""
@@ -176,6 +189,9 @@ class Client:
         """Make signed headers, use them to make a HTTP request of arbitrary form and return the result
         as decoded JSON. Optionally pass a payload to JSON-dump and parameters for the request call."""
 
+        if not body and requests_fn in [requests.patch]:
+            raise EmptyRequestError
+
         kwargs = {'params': params}
         None if body is None else kwargs.update({'data': json.dumps(body)})
         uri = urlparse(f'{self.credentials.api_endpoint}{path}')
@@ -784,12 +800,13 @@ class Client:
         :raises: :py:class:`~las.InvalidCredentialsException`, :py:class:`~las.TooManyRequestsException`,\
  :py:class:`~las.LimitExceededException`, :py:class:`requests.exception.RequestException`
         """
-        body = {
+        body = dictstrip({
             'groundTruth': ground_truth,
             'datasetId': dataset_id,
             'metadata': metadata,
-        }
-        return self._make_request(requests.patch, f'/documents/{document_id}', body=dictstrip(body))
+        })
+
+        return self._make_request(requests.patch, f'/documents/{document_id}', body=body)
 
     def delete_document(self, document_id: str) -> Dict:
         """Delete the document with the provided document_id, calls the DELETE /documents/{documentId} endpoint.
@@ -882,6 +899,7 @@ class Client:
         name: Optional[str] = None,
         description: Optional[str] = None,
         metadata: Optional[dict] = None,
+        base_model: Optional[dict] = None,
         **optional_args,
     ) -> Dict:
         """Creates a model, calls the POST /models endpoint.
@@ -902,12 +920,21 @@ class Client:
         :type description: str, optional
         :param metadata: Dictionary that can be used to store additional information
         :type metadata: dict, optional
+        :param base_model: Specify which model to use as base model. Example: \
+{"organizationId": "las:organization:cradl", "modelId": "las:model:invoice"}
+        :type base_model: dict, optional
         :return: Model response from REST API
         :rtype: dict
 
         :raises: :py:class:`~las.InvalidCredentialsException`, :py:class:`~las.TooManyRequestsException`,\
  :py:class:`~las.LimitExceededException`, :py:class:`requests.exception.RequestException`
         """
+        if base_model:
+            metadata = {
+                **(metadata or {}),
+                'baseModel': base_model,
+            }
+
         body = dictstrip({
             'width': width,
             'height': height,
@@ -921,13 +948,21 @@ class Client:
         body.update(**optional_args)
         return self._make_request(requests.post, '/models', body=body)
 
-    def list_models(self, *, max_results: Optional[int] = None, next_token: Optional[str] = None) -> Dict:
+    def list_models(
+        self,
+        *,
+        owner: Optional[Queryparam] = None,
+        max_results: Optional[int] = None,
+        next_token: Optional[str] = None,
+    ) -> Dict:
         """List models available, calls the GET /models endpoint.
 
         >>> from las.client import Client
         >>> client = Client()
         >>> client.list_models()
 
+        :param owner: Organizations to retrieve plans from
+        :type owner: Queryparam, optional
         :param max_results: Maximum number of results to be returned
         :type max_results: int, optional
         :param next_token: A unique token for each page, use the returned token to retrieve the next page.
@@ -941,8 +976,9 @@ class Client:
         params = {
             'maxResults': max_results,
             'nextToken': next_token,
+            'owner': owner,
         }
-        return self._make_request(requests.get, '/models', params=params)
+        return self._make_request(requests.get, '/models', params=dictstrip(params))
 
     def get_model(self, model_id: str) -> Dict:
         """Get a model, calls the GET /models/{modelId} endpoint.
@@ -1356,7 +1392,13 @@ class Client:
 
         return self._make_request(requests.get, f'/plans/{quote(plan_id, safe="")}')
 
-    def list_plans(self, *, owner: Optional[Queryparam] = None, max_results: Optional[int] = None, next_token: Optional[str] = None) -> Dict:
+    def list_plans(
+        self,
+        *,
+        owner: Optional[Queryparam] = None,
+        max_results: Optional[int] = None,
+        next_token: Optional[str] = None,
+    ) -> Dict:
         """List plans available, calls the GET /plans endpoint.
 
         >>> from las.client import Client
@@ -1376,9 +1418,9 @@ class Client:
     :py:class:`~las.LimitExceededException`, :py:class:`requests.exception.RequestException`
         """
         params = {
-            'owner': owner,
             'maxResults': max_results,
             'nextToken': next_token,
+            'owner': owner,
         }
         return self._make_request(requests.get, '/plans', params=dictstrip(params))
 
@@ -1586,8 +1628,9 @@ class Client:
         in_schema: Optional[dict] = None,
         out_schema: Optional[dict] = None,
         assets: Optional[dict] = None,
-        environment: Optional[dict] = None,
-        environment_secrets: Optional[list] = None,
+        cpu: Optional[int] = None,
+        memory: Optional[int] = None,
+        image_url: Optional[str] = None,
         **optional_args,
     ) -> Dict:
         """Updates a transition, calls the PATCH /transitions/{transitionId} endpoint.
@@ -1615,6 +1658,14 @@ class Client:
         :param environment_secrets: \
             A list of secretIds that contains environment variables to use for a docker transition
         :type environment_secrets: list, optional
+        :param cpu: Number of CPU units to use for a docker transition
+        :type cpu: int, optional
+        :param memory: Memory in MiB to use for a docker transition
+        :type memory: int, optional
+        :param image_url: Docker image url to use for a docker transition
+        :type image_url: str, optional
+        :param secret_id: Secret containing a username and password if image_url points to a private docker image
+        :type secret_id: str, optional
         :return: Transition response from REST API
         :rtype: dict
 
@@ -1624,10 +1675,24 @@ class Client:
         body = dictstrip({
             'inputJsonSchema': in_schema,
             'outputJsonSchema': out_schema,
-            'assets': assets,
-            'environment': environment,
-            'environmentSecrets': environment_secrets,
         })
+
+        parameters = dictstrip({
+            'assets': assets,
+            'cpu': cpu,
+            'imageUrl': image_url,
+            'memory': memory,
+        })
+
+        if 'environment' in optional_args:
+            parameters['environment'] = optional_args.pop('environment')
+        if 'environment_secrets' in optional_args:
+            parameters['environmentSecrets'] = optional_args.pop('environment_secrets')
+        if 'secret_id' in optional_args:
+            parameters['secretId'] = optional_args.pop('secret_id')
+        if parameters:
+            body['parameters'] = parameters
+
         body.update(**optional_args)
         return self._make_request(requests.patch, f'/transitions/{transition_id}', body=body)
 
@@ -1773,17 +1838,13 @@ class Client:
         :raises: :py:class:`~las.InvalidCredentialsException`, :py:class:`~las.TooManyRequestsException`,\
  :py:class:`~las.LimitExceededException`, :py:class:`requests.exception.RequestException`
         """
-        if isinstance(start_time, datetime):
-            if not start_time.tzinfo:
-                start_time = start_time.astimezone()
-            start_time = start_time.isoformat()
 
         url = f'/transitions/{transition_id}/executions/{execution_id}'
         body = {
             'status': status,
             'output': output,
             'error': error,
-            'startTime': start_time,
+            'startTime': datetimestr(start_time),
         }
         return self._make_request(requests.patch, url, body=dictstrip(body))
 
@@ -2077,14 +2138,16 @@ class Client:
         return self._make_request(requests.post, endpoint, body={'input': content})
 
     def list_workflow_executions(
-            self,
-            workflow_id: str,
-            *,
-            status: Optional[Queryparam] = None,
-            sort_by: Optional[str] = None,
-            order: Optional[str] = None,
-            max_results: Optional[int] = None,
-            next_token: Optional[str] = None,
+        self,
+        workflow_id: str,
+        *,
+        status: Optional[Queryparam] = None,
+        sort_by: Optional[str] = None,
+        order: Optional[str] = None,
+        max_results: Optional[int] = None,
+        next_token: Optional[str] = None,
+        from_start_time: Optional[Union[str, datetime]] = None,
+        to_start_time: Optional[Union[str, datetime]] = None,
     ) -> Dict:
         """List executions in a workflow, calls the GET /workflows/{workflowId}/executions endpoint.
 
@@ -2104,6 +2167,10 @@ class Client:
         :type max_results: int, optional
         :param next_token: A unique token for each page, use the returned token to retrieve the next page.
         :type next_token: str, optional
+        :param from_start_time: Specify a datetime range for start_time with from_start_time as lower bound
+        :type from_start_time: str or datetime, optional
+        :param to_start_time: Specify a datetime range for start_time with to_start_time as upper bound
+        :type to_start_time: str or datetime, optional
         :return: Workflow executions responses from REST API
         :rtype: dict
 
@@ -2118,6 +2185,11 @@ class Client:
             'maxResults': max_results,
             'nextToken': next_token,
         }
+
+        if any([from_start_time, to_start_time]):
+            params['fromStartTime'] = datetimestr(from_start_time)
+            params['toStartTime'] = datetimestr(to_start_time)
+
         return self._make_request(requests.get, url, params=params)
 
     def get_workflow_execution(self, workflow_id: str, execution_id: str) -> Dict:
